from .mana_pool import MANA_TYPES

class PayError(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)

def spell(mana_cost):
    """
    Decorator function to create spells!
    """
    def decorator(func):
        def wrap(*args, **kwargs):
            mana_pool = kwargs['mana_pool']
            pay = kwargs['mana_pay']
            kwargs.pop('mana_pool')
            kwargs.pop('mana_pay')

            cost = {}
            temp_mana_cost = mana_cost
            for mana_type in MANA_TYPES:
                cost[mana_type] = temp_mana_cost.count(mana_type)
                temp_mana_cost = temp_mana_cost.replace(mana_type, '')
            if temp_mana_cost != '':
                cost['generic'] = int(temp_mana_cost)
            else:
                cost['generic'] = 0

            generic_paid = 0
            temp_pay = dict(pay)
            for mana_type in MANA_TYPES:
                if mana_type in temp_pay:
                    temp_pay[mana_type] -= cost[mana_type]
                    if temp_pay[mana_type] < 0:
                        raise PayError("Not paying enough " + mana_type + "!")
                    generic_paid += temp_pay[mana_type]
            if generic_paid == cost['generic']:
                mana_pool.spend(pay)
            elif generic_paid > cost['generic']:
                raise PayError("Paying too much generic mana!")
            else:
                raise PayError("Not paying enough for generic mana!")

            func(*args, **kwargs)
        return wrap
    return decorator



MANA_TYPES = [
    'W',
    'U',
    'B',
    'R',
    'G'
]

DEFAULT_POOL = {
    'W': 0,
    'U': 0,
    'B': 0,
    'R': 0,
    'G': 0
}

LANDS = {
    'plains': 'W',
    'island': 'U',
    'swamp': 'B',
    'mountain': 'R',
    'forest': 'G'
}

class ManaError(Exception):
    def __init__(self, mana_type):
        Exception.__init__(self, "Not enough mana of type {" + mana_type + "}!")

class ManaPool:
    """
    You need this to cast spells!
    """
    def __init__(self):
        self.pool = DEFAULT_POOL

    def tap(self, land):
        self.pool[LANDS[land]] += 1
        return self

    def spend(self, pay):
        for mana_type in MANA_TYPES:
            if mana_type in pay:
                if self.pool[mana_type] - pay[mana_type] < 0:
                    raise ManaError(mana_type)
                else:
                    self.pool[mana_type] -= pay[mana_type]



from setuptools import setup

setup(
    name='pythonthegathering',
    packages=['pythonthegathering'],
    version='1.2.1',
    description='Replaces everything good and practical about Python with MTG!',
    author='Theo Hamilton/linky00',
    author_email='linky00@plotholestudios.com',
    url='https://github.com/linky00/pythonthegathering',
    download_url='https://github.com/linky00/pythonthegathering/archive/v1.2.tar.gz',
    keywords='decorators mtg',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],
    license='MIT'
)
